
import { type Logger } from 'winston';
import { multiaddrToUri } from '@multiformats/multiaddr-to-uri';
import https from 'https';
import fetch from 'node-fetch';
import { IPFS_CLUSTER_HTTP_API_MULTIADDR } from './env.js'

export async function pinCid (env: NodeJS.ProcessEnv, logger: Logger, cid: string): Promise<string> {
    const controller = new AbortController();
    const signal = controller.signal;
    const httpsAgent = new https.Agent({
        rejectUnauthorized: false,
    });
    const uri = multiaddrToUri(IPFS_CLUSTER_HTTP_API_MULTIADDR);

    logger.log({ level: 'debug', message: `uri=${uri}` });

    const duration: number = 60*1000*3;
    const timer = setTimeout(() => {
        logger.log({ level: 'debug', message: `aborting after ${duration}ms` })
        controller.abort();
    }, duration);

    const res = await fetch(`${uri}/pins/${cid}`, {
        signal,
        agent: httpsAgent,
        method: 'POST',
        headers: {
            'Authorization': `Basic ${Buffer.from(`${env.IPFS_CLUSTER_HTTP_API_USERNAME}:${env.IPFS_CLUSTER_HTTP_API_PASSWORD}`, "utf-8").toString("base64")}` 
        },
    });


    if (res.ok) {
        clearTimeout(timer); // Cancel the timeout

        const data = await res.json() as { cid: string; };
        logger.log({ level: 'debug', message: `data=${JSON.stringify(data)}` })

        if (data.cid !== cid) throw new Error(`CID did not match`);
        return data.cid
        
    } else {
        throw new Error(`fetch failed with status ${res.status} ${res.statusText}`);
    }
}

export async function isCidPinned (env: NodeJS.ProcessEnv, logger: Logger, cid: string): Promise<boolean> {
    const controller = new AbortController();
    const signal = controller.signal;
    const uri = multiaddrToUri(IPFS_CLUSTER_HTTP_API_MULTIADDR);

    logger.log({ level: 'debug', message: `uri=${uri}` });
    const httpsAgent = new https.Agent({
        rejectUnauthorized: false,
    });

    const duration: number = 60*1000;
    const timer = setTimeout(() => {
        logger.log({ level: 'debug', message: `aborting after ${duration}ms` })
        controller.abort();
    }, duration);

    const res = await fetch(`${uri}/pins/${cid}`, {
        signal,
        agent: httpsAgent,
        headers: {
            'Authorization': `Basic ${Buffer.from(`${env.IPFS_CLUSTER_HTTP_API_USERNAME}:${env.IPFS_CLUSTER_HTTP_API_PASSWORD}`, "utf-8").toString("base64")}` 
        },
    });

    if (res.ok) {
        clearTimeout(timer); // Cancel the timeout

        const data = await res.json();
        logger.log({ level: 'debug', message: `data=${JSON.stringify(data)}` })
        return (isCidPinnedOnPeers(data, cid))

    } else {
        return false; // Fetch failed
    }

}


function isCidPinnedOnPeers(clusterData: any, targetCid: string): boolean {
    const peerMap = clusterData.peer_map;

    // Extract the CID from the top-level clusterData
    const topLevelCid = clusterData.cid;

    for (const peerId in peerMap) {
        if (peerMap.hasOwnProperty(peerId)) {
            const peerInfo = peerMap[peerId];

            // Check if the peer has the target CID pinned
            if (peerInfo.status === 'pinned' && topLevelCid === targetCid) {
                return true; // CID is pinned on at least 1 peer
            }
        }
    }

    return false; // CID is not pinned on any peer
}

