import { IRunTaskFunction, ITaskOptions } from "../Task.js";
import { PythonShell, Options } from 'python-shell';
import path, { dirname } from 'path';
import { fileURLToPath } from 'url';
import { TWITTER_ACCOUNTS, STRAPI_URL, STRAPI_API_KEY } from '../env.js';
import { ITweetScrape, IStrapiTweet, isChaturbateInviteLinkPresent, createTweet } from '../tweets.js';
import { createStream } from "../stream.ts";
import { Logger } from "winston";
import qs from 'qs';

const __dirname = dirname(fileURLToPath(import.meta.url));



export const name = 'crawlTwitterMonth';
/**
 * search on twitter 
 * 
 * Get tweets for a given month.
 * For each tweet, look for a chaturbate invite link
 * If such a link is present, create a livestream entry in the db
 * 
 * 
 */
export const runTask: IRunTaskFunction = async ({ logger, env, job }: ITaskOptions) => {
    logger.log({ level: 'info', message: `🀄 crawlTwitterMonth begins. since=${job.data?.since}, until=${job.data?.until}, username=${job.data?.username}` });

    if (!TWITTER_ACCOUNTS) throw new Error('TWITTER_ACCOUNTS was undefined in env, but it is required.');
    if (!job.data?.since) throw new Error('job.data.since was undefined');
    if (!job.data?.until) throw new Error('job.data.until was undefined');
    if (!job.data?.username) throw new Error('job.data.username was undefined');
    if (!job.data?.vtuberId) throw new Error('job.data.vtuberId was undefined, but it is required.');

    const twitterAccounts = TWITTER_ACCOUNTS.split(',');
    const scriptPath = path.join(__dirname, '..', '..', 'python_scripts');
    const pythonPath = path.join(__dirname, '..', '..', 'python_scripts', 'venv', 'bin', 'python3');
    logger.log({ level: 'debug', message: `__dirname=${__dirname} import.meta.url=${import.meta.url}, scriptPath=${scriptPath}` })

    const opts: Options = {
        mode: "json",
        pythonPath: pythonPath,
        scriptPath: scriptPath,
        args: [
            '--accounts', ...twitterAccounts,
            '--search_query', `(from:${job.data.username}) since:${job.data.since} until:${job.data.until}`
        ]
    }

    const tweets: ITweetScrape[] = await PythonShell.run('twitterScraper.py', opts);

    await Promise.all(tweets.map(async (tweet) => {
        const tweetInStrapi = await createTweet(logger, tweet, job.data.vtuberId); // assert tweet's existence
        const isInviteTweet = await isChaturbateInviteLinkPresent(tweet);
        if (isInviteTweet) {
            logger.log({ level: 'debug', message: `Tweet contains CB invite link. One sec while I create a stream entry in Strapi.` });
            const streamInStrapi = await createStream(logger, tweetInStrapi, job.data.vtuberId);
            logger.log({ level: 'debug', message: `streamInStrapi id=${streamInStrapi.id}` });
        } else {
            logger.log({ level: 'debug', message: `tweet ${tweet.id_str} is NOT an invite tweet`})
        }
    }))

    


    logger.log({ level: 'info', message: `🀄 crawlTwitterMonth ends. Processed ${tweets.length} tweets.` });


}