

import Prevvy from 'prevvy';
import path from 'node:path';
import { uploadToB2 } from '../b2.js'
import { IVod, getVod } from '../vods.js';
import { IJobData } from '../types.js';
import { IB2File } from '../b2File.js';
import { Logger } from 'winston';
import { IRunTaskFunction, ITaskOptions } from '../Task.js';
import { Job } from 'bullmq';
import { TMPDIR } from '../env.js';

export interface IUploadData {
  key: string;
  uploadId: string;
  url: string;
}


export const name = 'generateThumbnail';

export async function __generateThumbnail(vod: IVod, env: NodeJS.ProcessEnv, logger: Logger, job: Job): Promise<string> {
  const fileName = `vod-${vod?.id}-thumb.png`;
  const thumbnailFilePath = path.join(TMPDIR, fileName);
  const videoInputUrl = vod.attributes.videoSrcB2?.data?.attributes?.cdnUrl;
  if (!videoInputUrl) {
    console.error(vod?.attributes?.videoSrcB2);
    throw new Error(`videoInputUrl in __generateThumbnail was undefined`);
  }
  logger.log({ level: 'info', message: `🫰 Creating thumbnail from ${videoInputUrl} ---> ${thumbnailFilePath}` });

  const thumb = new Prevvy({
    input: videoInputUrl,
    output: thumbnailFilePath,
    throttleTimeout: 2000,
    width: 128,
    cols: 5,
    rows: 5,
  });

  thumb.on('progress', async (data: { percentage: number }) => {
    await job.updateProgress(data.percentage);
  });

  await thumb.generate();
  return thumbnailFilePath;

}

export async function associateB2WithVod(vod: IVod, uploadData: IUploadData, env: NodeJS.ProcessEnv, logger: Logger) {
  logger.log({ level: 'info', message: `🥤 lets create b2-file in Strapi` });

  // Create the B2 file
  const thumbResponse = await fetch(`${env.STRAPI_URL}/api/b2-files`, {
    method: 'POST',
    headers: {
      'Authorization': `Bearer ${env.STRAPI_API_KEY}`,
      'Content-Type': 'application/json',
    },
    body: JSON.stringify({
      data: {
        key: uploadData.key,
        uploadId: uploadData.uploadId,
        url: uploadData.url,
        cdnUrl: `https://futureporn-b2.b-cdn.net/${uploadData.key}`
      },
    }),
  });

  if (!thumbResponse.ok) {
    const msg = `🟠 Failed to create B2 file: ${thumbResponse.statusText}`
    console.error(msg)
    throw new Error(msg);
  }

  const thumbData = await thumbResponse.json() as IB2File;

  logger.log({ level: 'info', message: `📀 B2 file creation complete for B2 file id: ${thumbData.data.id}` });

  logger.log({ level: 'info', message: `🪇 lets associate B2-file with VOD ${vod.id} in Strapi` });

  // Associate B2 file with VOD
  const associateResponse = await fetch(`${env.STRAPI_URL}/api/vods/${vod.id}`, {
    method: 'PUT',
    headers: {
      'Authorization': `Bearer ${env.STRAPI_API_KEY}`,
      'Content-Type': 'application/json',
    },
    body: JSON.stringify({
      data: {
        thumbnail: thumbData.data.id,
      },
    }),
  });

  if (!associateResponse.ok) {
    const msg = `💀 Failed to associate B2 file with VOD: ${associateResponse.statusText}`;
    console.error(msg)
    throw new Error(msg)
  }

  logger.log({ level: 'info', message: `🫚 Association complete` });
}



export const runTask: IRunTaskFunction = async ({ logger, env, job }: ITaskOptions) => {
  const data = job.data as IJobData;
  if (!data.id) {
    const msg = 'panic! no vod id was passed to generateThumbnail.'
    console.error(msg)
    throw new Error(msg)
  }
  const vod = await getVod(data.id, env);
  if (!vod) {
    const msg = `panic! vod ${data.id} missing`
    console.error(msg)
    throw new Error(msg)
  }

  logger.log({ level: 'info', message: '🖼️ __generateThumbnail begin' })
  const thumbnailFilePath = await __generateThumbnail(vod, env, logger, job);

  logger.log({ level: 'info', message: `🆙 uploading thumbnail ${thumbnailFilePath} for vod ${data.id} to B2` });
  const uploadData = await uploadToB2(env, thumbnailFilePath);
  if (!uploadData) {
    const msg = 'panic! uploadData missing'
    console.error(msg)
    throw new Error(msg);
  }

  logger.log({ level: 'info', message: `👍 associating thumbnail for vod ${data.id} with strapi` })
  await associateB2WithVod(vod, uploadData, env, logger)
  logger.log({ level: 'info', message: `👍👍 thumbnail associated with vod ${data.id}` });

  return 'DONE'
}