import { getVod, IVod, getRandomVod } from "../vods.js";
import fetch, { AbortError } from 'node-fetch';
import { IJobData } from "../types.js";
import { ITaskOptions, IRunTaskFunction, IIssueDefinition } from "../Task.js";
import { Queue } from 'bullmq';
import { isCidPinned } from '../ipfsCluster.js';
import { Logger } from "winston";


export const name = 'identifyVodIssues';

/*
    The check functions return true if there is an issue
    This is an exhaustive list of all possible problems that can exist on a vod.
 */
const issueDefinitions: IIssueDefinition[] = [
    {
        name: 'thumbnailMissing',
        check: async (env, logger, vod) => {
            if (!vod?.attributes?.thumbnail?.data?.attributes?.cdnUrl) return true;
            else return false;
        },
        solution: 'generateThumbnail'
    },
    {
        name: 'thumbnailUnreachable',
        check: async (env, logger, vod) => {
            if (!vod?.attributes?.thumbnail?.data?.attributes?.cdnUrl) return false; // false because the problem isn't explicitly that the thumb is unreachable
            const response = await fetch(vod.attributes.thumbnail.data.attributes.cdnUrl);
            if (!response.ok) return true;
            else return false;
        },
        solution: 'deleteThumbnail'
    },
    // {
    // // WARNING: Mux quickly becomes unaffordable.
    // // @todo implement human-in-the-loop encoding approval
    //     name: 'muxAssetMissing',
    //     check: async (vod) => {
    //         // we only want to allocate new videos
    //         // so we only consider vods published after
    //         // a certain date
    //         const allocationCutoffDate = new Date('2023-06-01T00:00:00.000Z');
    //         const vodDate = new Date(vod.date2);
    //         const isVodOld = isBefore(vodDate, allocationCutoffDate)
    //         logger.log({ level: 'info', message: `muxAsset:${vod?.muxAsset?.assetId}, vodDate:${vod.date2}, allocationCutoffDate:${allocationCutoffDate.toISOString()}, isVodOld:${isVodOld}` })
    //         if (isVodOld) return false;
    //         if (!!vod?.muxAsset?.assetId) return false;
    //         logger.log({ level: 'error', message: `vod ${vod.id} is missing a muxAsset!` })
    //         return true;
    //     },
    //     solution: 'createMuxAsset'
    // },
    {
        name: 'videoSrcHashUnpinned',
        check: async (env: NodeJS.ProcessEnv, logger: Logger, vod: IVod) => {
            const isPinned = await isCidPinned(env, logger, vod.attributes.videoSrcHash);
            return (!isPinned);
        },
        solution: 'pinVideoSrcHash'
    },
    // { 
    //     name: 'videoSrcHashUnreachable',
    //     check: async (vod, env) => {
    //         // /api/v0/pin/add the CID to see if it takes long or not.
    //         // if the pin add is taking > 5 minutes, consider the CID not present on the provider
    //         // if the pin is not present, get the file from backup
    //         // and then /api/v0/add the file to IPFS.
    //         // @todo
    //         const res = fetch(`${uri}/api/v0/pin/add?arg=${vod.videoSrcHash}&progress=1`, {
    //             method: 'POST'
    //         })
    //         return false;
    //     },
    //     solution: 'pinVideoSrcHashFromBackup'
    // },


    // {
    //     name: 'video240HashUnreachable',
    //     check: async (vod) => {

    //     },
    //     solution: 'pinFromBackup'
    // }
]



export const runTask: IRunTaskFunction = async ({ logger, env, job, connection }: ITaskOptions) => {

    logger.log({ level: 'debug', message: `🆕 identifyVodIssues begin` });

    const data = job.data as IJobData;
    let vod: IVod | null;

    // determine if we received a vod id or if we need to choose a random vod
    if (!data?.id) {
        // get a random vod
        vod = await getRandomVod(env);
    } else {
        // get a vod by id
        vod = await getVod(data.id, env);
    }

    if (!vod) {
        const msg = '☠️ Panic! Could not get a vod';
        logger.log({ level: 'error', message: msg });
        throw new Error(msg);
    }

    const id = vod?.id;
    if (!vod.id) throw new Error(`vod was missing ID`);

    logger.log({ level: 'info', message: `📹 VOD ${id}` });
    for (const iDef of issueDefinitions) {
        const isProblem = await iDef.check(env, logger, vod);
        const status = isProblem ? '🔴 FAIL' : '🟢 PASS';
        logger.log({ level: 'info', message: `${status} ${iDef.name} ${isProblem ? `(queueing ${iDef.solution} to solve.)` : ''}` });
        if (isProblem) {
            logger.log({ level: 'info', message: `🚩 VOD Problem detected. Adding ${iDef.solution} task to the queue.` })
            const queue = new Queue(iDef.solution, {
                connection
            })
            queue.add(iDef.solution, { id: vod.id });
        }
    }
    return 'OK'
}

