import fetch from 'node-fetch';
import { Logger } from 'winston';
import qs from 'qs';
import { STRAPI_API_KEY, STRAPI_URL } from './env.ts';

const chaturbateLinkRegex = /chaturbate.com/;

/**
 * This link map exists because some of
 * projektmelody's repeatedly-used shorturls
 * are expired as of Nov 2023.
 * Instead of redirecting to her chaturbate, 
 * The visitor is redirected to centralreach.com
 * 
 * To handle these broken links, we're taking note of
 * the links which are embedded in old tweets.
 * any instance of these specific links equates to a CB invite link.
 */
export const brokenLinkMap = new Map([
    ['http://shorturl.at/tNUVY', 'https://chaturbate.com/projektmelody']
]);

export interface IStrapiTweet {
    id: number;
    attributes: {
        id_str: string;
        url: string;
        date: string;
        json: string;
    }
}

export interface IMeta {
    pagination: {
        total: number;
    }
}

export interface IStrapiTweetResponse {
    data: IStrapiTweet[];
    meta: IMeta;
}

export interface IStrapiTweetSingleResponse {
    data: IStrapiTweet;
    meta: IMeta;
}

export interface ITweetScrape {
    id: number;
    id_str: string;
    url: string;
    date: string;
    user: {
        id: number;
        id_str: string;
        url: string;
        username: string;
        displayname: string;
        rawDescription: string;
        created: string;
        followersCount: number;
        friendsCount: number;
        statusesCount: number;
        favouritesCount: number;
        listedCount: number;
        mediaCount: number;
        location: string;
        profileImageUrl: string;
        profileBannerUrl: string;
        protected: boolean | null;
        verified: boolean;
        blue: boolean;
        blueType: string | null;
        descriptionLinks: {
            url: string;
            text: string;
            tcourl: string;
        }[];
        _type: string;
    };
    lang: string;
    rawContent: string;
    replyCount: number;
    retweetCount: number;
    likeCount: number;
    quoteCount: number;
    conversationId: number;
    hashtags: string[];
    cashtags: string[];
    mentionedUsers: any[];
    links: {
        url: string;
        text: string;
        tcourl: string;
    }[];
    viewCount: number;
    retweetedTweet: any;
    quotedTweet: any;
    place: any;
    coordinates: any;
    inReplyToTweetId: any;
    inReplyToUser: any;
    source: string;
    sourceUrl: string;
    sourceLabel: string;
    media: {
        photos: {
            url: string;
        }[];
        videos: any[];
        animated: any[];
    };
    _type: string;
}


export async function createTweet (logger: Logger, tweet: ITweetScrape, vtuberId: number): Promise<IStrapiTweet> {
    logger.log({ level: 'debug', message: `Searching for tweet with same id. id_str=${tweet.id_str} vtuberId=${vtuberId}` });
    if (!vtuberId) throw new Error(`createTweet was missing vtuberId argument, which is required.`);
    const existingTweetQuery = qs.stringify({
        filters: {
            id_str: {
                $eq: tweet.id_str
            }
        }
    })
    const existingTweetRes = await fetch(`${STRAPI_URL}/api/tweets?${existingTweetQuery}`, {
        method: 'GET',
        headers: {
            'authorization': `Bearer ${STRAPI_API_KEY}`,
            'content-type': 'application/json'
        },
    });
    const existingTweetJson = await existingTweetRes.json() as IStrapiTweetResponse;
    if (!!existingTweetJson.data[0]?.attributes?.id_str) return existingTweetJson.data[0];

    // Create tweet if it isnt already saved in Strapi
    const tweetRes = await fetch(`${STRAPI_URL}/api/tweets`, {
        method: 'POST',
        headers: {
            'authorization': `Bearer ${STRAPI_API_KEY}`,
            'content-type': 'application/json'
        },
        body: JSON.stringify({
            data: {
                date: new Date(tweet.date).toISOString(),
                id_str: tweet.id_str,
                url: tweet.url,
                json: JSON.stringify(tweet),
                vtuber: vtuberId
            }
        })
    })
    const bod = await tweetRes.json() as IStrapiTweetSingleResponse;
    if (!tweetRes.ok) {
        throw new Error(`failed to create tweet entry in Strapi. status=${tweetRes.status}, statusText=${tweetRes.statusText} body=${bod}`);
    }

    logger.log({ level: 'info', message: `Created tweet in Strapi. bod=${JSON.stringify(bod)}` });

    return bod.data;
}

/*
 * expands a shortened url such as those from tinyUrl or other URL shortening service.
 * 
 */
export async function expandUrl(url: string, redirects: number = 0): Promise<string> {
    if (!url) throw new Error('expandUrl expects URL as an argument, but it was empty.');
    if (!URL.canParse(url)) return url;
    if (brokenLinkMap.has(url)) return brokenLinkMap.get(url)!;
    if (chaturbateLinkRegex.test(url)) return url;

    if (redirects > 19) {
        throw new Error(`Too many redirects while expanding url=${url}`);
    }

    try {
        const res = await fetch(url, {
            method: 'HEAD',
            redirect: 'manual'
        });

        if ([301, 302, 303, 308].includes(res.status)) {
            const location = res.headers.get('location');
            if (!location) throw new Error('Failed to get location header');
            else return expandUrl(location, redirects + 1);
        } else {
            // We return the input url in case of 404s or any other non-redirect scenario.
            return url;
        }
    } catch (e) {
        // It's better to return the original input URL than it is to crash.
        return url;
    }
}




/**
 * 
 * Looks at all the URLs in the tweet, expanding them (from link shortener) if necessary
 * Resolves true if there are any chaturbate invite links in the tweet.
 * 
 */
export async function isChaturbateInviteLinkPresent(tweet: ITweetScrape): Promise<boolean> {
    const expandedLinks = await Promise.all(tweet.links.map(async (link) => {
        const expandedLink = await expandUrl(link.url);
        return expandedLink;
    }));
    return expandedLinks.some(isChaturbateLink);
}


export function isChaturbateLink(link: string): boolean {
    return (chaturbateLinkRegex.test(link));
}