
import { strapiUrl } from './constants.js'
import { getDateFromSafeDate, getSafeDate } from './dates.js'
import { IVtuber } from './vtubers.js'
import qs from 'qs'
import { ITagVodRelation, unmarshallTagVodRelation } from './tag-vod-relations.js'
import { IB2File } from './b2File.js'
import fetch from 'node-fetch'

export interface IMuxAsset {
    playbackId: string;
    assetId: string;
}


export interface IVods {
    data: IVod[];
    pagination: {
        page: number;
        pageSize: number;
        pageCount: number;
        total: number;
    }
}

export interface IVodsResponse {
    data: {
        attributes: {
            pledge_sum: number;
            patron_count: number;
        }
    }
    meta: {
        pagination: {
            total: number;
        }
    }
}

export interface IVod {
    id: number;
    attributes: {
        title?: string;
        date: string;
        date2: string;
        muxAsset: IMuxAsset;
        thumbnail: IB2File | null;
        vtuber: IVtuber;
        tagVodRelations: ITagVodRelation[];
        video240Hash: string;
        videoSrcHash: string;
        videoSrcB2: IB2File | null;
        announceTitle: string;
        announceUrl: string;
        note: string;
    }
}






export async function getVodForDate(date: Date): Promise<IVod> {
    const iso8601DateString = date.toISOString().split('T')[0];
    const query = qs.stringify(
        {
            filters: {
                date: {
                    $eq: date.toISOString()
                }
            },
            populate: {
                vtuber: {
                    fields: ['slug', 'displayName', 'image', 'imageBlur', 'themeColor']
                },
                muxAsset: {
                    fields: ['playbackId', 'assetId']
                },
                thumbnail: {
                    fields: ['cdnUrl', 'url']
                },
                tagVodRelations: {
                    fields: ['tag'],
                    populate: ['tag']
                },
                videoSrcB2: {
                    fields: ['url', 'key', 'uploadId', 'cdnUrl']
                }
            },
            // populate: '*'
        }
    )
    return fetch(`${strapiUrl}/api/vods?${query}`)
        .then((res) => res.json())
        .then((d: any) => d.data[0])
}

export async function getRandomVod(env: NodeJS.ProcessEnv): Promise<IVod | null> {
    try {
        const d = await fetch(`${strapiUrl}/api/vod/random`)
            .then((res) => res.json()) as IVod;
        return d;
    } catch (e) {
        console.error(`There was an error while fetching random vod`);
        console.error(e);
        return null;
    }
}


export async function getVod(id: number, env: NodeJS.ProcessEnv): Promise<IVod | null> {
    const query = qs.stringify(
        {
            filters: {
                id: {
                    $eq: id
                }
            },
            populate: {
                vtuber: {
                    fields: ['slug', 'displayName', 'image', 'imageBlur']
                },
                muxAsset: {
                    fields: ['playbackId', 'assetId']
                },
                thumbnail: {
                    fields: ['cdnUrl', 'url']
                },
                tagVodRelations: {
                    fields: ['tag'],
                    populate: ['tag']
                },
                videoSrcB2: {
                    fields: ['url', 'key', 'uploadId', 'cdnUrl']
                }
            }
        }
    )
    try {
        const res: any = await fetch(`${strapiUrl}/api/vods?${query}`)
        const data: IVod = await res.json();

        return data;
    } catch (e) {
        console.error(`there was an error while fetching vod ${id}`);
        console.error(e);
        return null;
    }
}


// export async function getVod(id: number, env: NodeJS.ProcessEnv): Promise<IVod | null> {
//     const rawVod = await getRawVod(id, env);
//     return unmarshallVod(rawVod);
// }

export async function getVods(page: number = 1, pageSize: number = 25, sortDesc = true): Promise<IVods> {
    const query = qs.stringify(
        {
            populate: {
                vtuber: {
                    fields: ['slug', 'displayName', 'image', 'imageBlur']
                },
                muxAsset: {
                    fields: ['playbackId', 'assetId']
                },
                thumbnail: {
                    fields: ['cdnUrl', 'url']
                },
                tagVodRelations: {
                    fields: ['tag'],
                    populate: ['tag']
                },
                videoSrcB2: {
                    fields: ['url', 'key', 'uploadId', 'cdnUrl']
                }
            },
            sort: {
                date: (sortDesc) ? 'desc' : 'asc'
            },
            pagination: {
                pageSize: pageSize,
                page: page
            }
        }
    )
    const res = await fetch(`${strapiUrl}/api/vods?${query}`);
    const vods = await res.json() as IVods
    return vods;
}






/**
 * This returns stale data, because futureporn-historian is broken.
 * @todo get live data from historian
 * @see https://git.futureporn.net/futureporn/futureporn-historian/issues/1
 */
export async function getProgress(vtuberSlug: string): Promise<{ complete: number; total: number }> {
    const query = qs.stringify({
        filters: {
            vtuber: {
                slug: {
                    $eq: vtuberSlug
                }
            }
        }
    })
    const data = await fetch(`${strapiUrl}/api/vods?${query}`)
        .then((res) => res.json() as any)

    const total: number = data.meta.pagination.total

    return {
        complete: total,
        total: total
    }
}