import { describe } from "node:test"
import chai, { expect } from 'chai';
import chaiAsPromised from 'chai-as-promised';
import { expandUrl, isChaturbateInviteLinkPresent,　isChaturbateLink } from "../src/tweets.js"
import nock from 'nock';
import tweetExpiredCb from './fixtures/tweetExpiredCb.json' assert { type: 'json' };
import tweetCb from './fixtures/tweetCb.json' assert { type: 'json' };
import tweetFansly from './fixtures/tweetFansly.json' assert { type: 'json' };
import tweetLinkless from './fixtures/tweetLinkless.json' assert { type: 'json' };
import tweetTwitch from './fixtures/tweetTwitch.json' assert { type: 'json' };

chai.use(chaiAsPromised);

describe('tweets', function () {
    describe('isChaturbateInviteLinkPresent', function () {
        it('should load json fixtures properly', function () {
            expect(tweetExpiredCb).to.have.property('_type', 'snscrape.modules.twitter.Tweet');
            expect(tweetCb).to.have.property('_type', 'snscrape.modules.twitter.Tweet');
            expect(tweetFansly).to.have.property('_type', 'snscrape.modules.twitter.Tweet');
            expect(tweetLinkless).to.have.property('_type', 'snscrape.modules.twitter.Tweet');
            expect(tweetTwitch).to.have.property('_type', 'snscrape.modules.twitter.Tweet');
        })
        it('should resolve true to a tweet with link to expired url shortener redirecting to cb', function () {
            return expect(isChaturbateInviteLinkPresent(tweetExpiredCb)).to.eventually.be.true;
        })
        it('should resolve true to a tweet with link to a cb room', function () {
            return expect(isChaturbateInviteLinkPresent(tweetCb)).to.eventually.be.true;
        })
        it('should resolve false to a tweet with link to a fansly room', function () {
            return expect(isChaturbateInviteLinkPresent(tweetFansly)).to.eventually.be.false;
        })
        it('should resolve false for a tweet with no links', function () {
            return expect(isChaturbateInviteLinkPresent(tweetLinkless)).to.eventually.be.false;
        })
        it('should resolve false for a tweet with a twitch link', function() {
            return expect(isChaturbateInviteLinkPresent(tweetTwitch)).to.eventually.be.false;
        })
    })

    describe('isChaturbateLink', function () {
        it('should detect chaturbate.com/b/:room/', function () {
            const link = 'https://chaturbate.com/b/projektmelody/'
            expect(isChaturbateLink(link)).to.be.true;
        })
        it('should detect chaturbate.com/:room/', function () {
            const link = 'https://chaturbate.com/projektmelody/'
            expect(isChaturbateLink(link)).to.be.true;
        })
    })

    describe('expandUrl', function () {
        it('should handle an invalid URL', function () {
            const invalid = '/users/198128/artworks';
            return expect(expandUrl(invalid)).to.eventually.equal(invalid);
        })
        it('should handle a 404', function () {
            const jast = 'https://jast.us/melody';
            return expect(expandUrl(jast)).to.eventually.equal(jast);
        })
        it('should expand tinyurl link', async function () {
            const shortUrl = new URL('https://tinyurl.com/58p5k9ru');
            const expectedUrl = 'https://www.tacobell.com/';
            nock(shortUrl.origin)
                .head(shortUrl.pathname)
                .reply(308, 'mocked by nock', {
                    'location': expectedUrl
                })
            const expandedUrl = await expandUrl(shortUrl.toString());
            expect(expandedUrl).to.equal(expectedUrl);
        })
        it('should expand shorturl link', async function () {
            const shortUrl = new URL('https://www.shorturl.at/ejlLT');
            const expectedUrl = 'https://www.tacobell.com/';
            nock(shortUrl.origin)
                .head(shortUrl.pathname)
                .reply(302, 'mocked by nock', {
                    'location': expectedUrl
                })
            const expandedUrl = await expandUrl(shortUrl.toString());
            expect(expandedUrl).to.equal(expectedUrl);
        })
        it('should return an already expanded link', async function () {
            const expandedUrl = 'https://chaturbate.com/b/projektmelody/';
            const doubleExpandedUrl = await expandUrl(expandedUrl);
            expect(doubleExpandedUrl).to.equal(expandedUrl);
        })
    })
})
